# Copyright (c) 2010, 2011, Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Tests for the date-directory based repository."""

__metaclass__ = type

import datetime
import os.path
import stat

from fixtures import TempDir
from pytz import utc
import testtools

from oops_datedir_repo import DateDirRepo
from oops_datedir_repo.uniquefileallocator import UniqueFileAllocator


class TestDateDirRepo(testtools.TestCase):

    def test_publish_permissions(self):
        errordir = self.useFixture(TempDir()).path
        repo = DateDirRepo(errordir, 'T')
        report = {'id': 'OOPS-91T1'}
        now = datetime.datetime(2006, 04, 01, 00, 30, 00, tzinfo=utc)

        # Set up default file creation mode to rwx------ as some restrictive
        # servers do.
        umask_permission = stat.S_IRWXG | stat.S_IRWXO
        old_umask = os.umask(umask_permission)
        self.addCleanup(os.umask, old_umask)
        repo.publish(report, now)

        errorfile = os.path.join(repo.log_namer.output_dir(now), '01800.T1')
        self.assertTrue(os.path.exists(errorfile))

        # Check errorfile is set with the correct permission: rw-r--r--
        st = os.stat(errorfile)
        wanted_permission = (
            stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
        # Get only the permission bits for this file.
        file_permission = stat.S_IMODE(st.st_mode)
        self.assertEqual(file_permission, wanted_permission)

    def test_sets_log_namer_to_a_UniqueFileAllocator(self):
        repo = DateDirRepo(self.useFixture(TempDir()).path, 'T')
        self.assertIsInstance(repo.log_namer, UniqueFileAllocator)
